/*
デバイスファイルをOpen/Closeするモジュール
2014/07/14	by S.Ushimaru
*/
#include <linux/errno.h>
#include <sys/io.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>
#include <fcntl.h>
#include "device.h"

#include "i2c.h"
#include "ui.h"
#include "compass.h"


static int fd_encoder;
static int fd_motor;
static int fd_uss;
static int fd_io;

/* 基本文字設定 */
const UIFont normalFont = {0,0,0,{UI_BLACK,UI_WHITE}};

/* 終了処理 */
void finalize();
/* タイトル表示 */
void printTitle();
/* 目盛り */
void printScale(int max);
/* プログレスバー */
void printGuage(double deg);

I2CBus* bus;
	Compass* compass;
	FILE* pipeInput;
	char busName[128];
	char input[128];
	int busCount,deviceCount;
	int address;
	int offsetCols;
	int intervalTime;
	int i;

int device_encoder_open(){

        //fd_encoder=open("/dev/mg3_encoder_dev",O_RDWR);
        fd_encoder=open("/dev/encoder_dev",O_RDWR);
        printf("fd_encoder=%d\n",fd_encoder);
        if(fd_encoder==-1){
                printf("encoder_dev can't open !!!\n");
                return -1;
        }
                return 0;

}

int device_motor_open(){

        //fd_motor=open("/dev/mg3_motor_dev",O_RDWR);
        fd_motor=open("/dev/motor_dev",O_RDWR);
        printf("fd_motor=%d\n",fd_motor);
        if(fd_motor==-1){
                printf("motor_dev can't open !!!\n");
                return -1;
        }
                return 0;
}

int device_uss_open(){

	//fd_uss=open(S_PORT,O_RDWR | O_NOCTTY | O_NONBLOCK);
	fd_uss=open("/dev/ttyS0",O_RDWR | O_NOCTTY | O_NONBLOCK);
        printf("fd_uss=%d\n",fd_uss);
        if(fd_uss==-1){
                printf("uss_dev can't open !!!\n");
                return -1;
        }
                return 0;
}

int device_io_open(){

	//fd_io=open("/dev/mg3_io_dev",O_RDONLY);
	fd_io=open("/dev/io_dev",O_RDONLY);
        printf("fd_io=%d\n",fd_io);
        if(fd_io==-1){
                printf("io device can't open !!!\n");
                return -1;
        }
                return 0;
}

int device_compass_open(){


	/* I2Cバスの検索 */
	uiSetFont(normalFont);
	printf("I2C bus list");
	uiMoveCols(offsetCols); printf(": ");
	busCount = 0;
	pipeInput = popen("ls /dev/ttyUSB* 2> /dev/null","r");
	while( fgets(busName,sizeof(busName),pipeInput)!=NULL ){
		busName[strlen(busName)-1] = '\0';
		printf("%s ",busName);
		busCount++;
	}
	if(busCount == 0){
		printf("ERROR\n");
		fprintf(stderr,"ERROR: No I2C bus");
		exit(1);
	}else{
		printf("\n");
	}

	/* バスのオープン */
	
	printf("I2C bus open");
	bus = i2cBusOpen(busName);
	
	address = 0;
	deviceCount = 0;
	for(i=0; i<=I2C_ADDRESS_MAX; i++){
		if( i2cBusHasDevice(bus,i) ){
			address = i;
			printf("0x%02X ",address);
			deviceCount++;
		}
	}
	if(deviceCount == 0){
		printf("ERROR\n");
		fprintf(stderr,"ERROR: No Device\n");
		i2cBusClose(bus);
		exit(1);
	}else{
		printf("\n");
	}

	/* 電子コンパスオープン */
	if(deviceCount > 1){
		printf("compass address");
		uiMoveCols(offsetCols); printf("? ");
		printf("0x");
		if( fgets(input,sizeof(input),stdin) == NULL){
			printf("ERROR\n");
			exit(1);
		}
		address = strtol(input,NULL,16);
	}
	printf("compass open");
	compass = compassOpen(bus,address);
	if(compass == NULL){
		printf("ERROR\n");
		i2cBusClose(bus);
		exit(1);
	}else{
		printf("OK\n");
	}

}


int device_encoder_close(){
		close(fd_encoder);
                return 0;
}
int device_motor_close(){
		close(fd_motor);
                return 0;
}
int device_uss_close(){
		close(fd_uss);
                return 0;
}
int device_io_close(){
		close(fd_io);
                return 0;
}
int device_compass_close(){
	printf("compass close");
	uiMoveCols(offsetCols); printf(": ");
	compassClose(compass);
	printf("OK\n");

	/* バスクローズ */
	printf("I2C bus close");
	uiMoveCols(offsetCols); printf(": ");
	i2cBusClose(bus);
	printf("OK\n");

                return 0;
}


int device_encoder_get(){
	return fd_encoder;
}
int device_motor_get(){
	return fd_motor;
}
int device_uss_get(){
	return fd_uss;
}

int device_io_get(){
	return fd_io;
}

