/*
 * compass.h
 *
 *  Created on: 2013/12/28
 *      Author: d10130
 *
 * 電子コンパス(HMC6352)との通信用ライブラリ
 */

#ifndef COMPASS_H_
#define COMPASS_H_

#include <stdint.h>
#include "i2c.h"

/*---- コンパス構造体 ----*/
struct Compass_;
typedef struct Compass_ Compass;

/*---- 方位の取得間隔 [usec] ----*/
#define COMPASS_INTERVAL_TIME 5000

/*
 * コンパスとの通信を開始する
 * 引数: I2Cバス,アドレス
 * 戻値: 正常時 コンパスポインタ
 *       異常時 NULL (errnoで判断)
 */
Compass* compassOpen(I2CBus* bus,uint8_t address);

/*
 * コンパスとの通信を終了する
 * 引数: コンパスポインタ
 */
void compassClose(Compass* compass);

/*
 * 初期値からの旋回角(半時計方向)を取得する
 * 引数: コンパスポインタ
 * 戻値: 正常時 測定結果 -M_PI ~ M_PI [rad]
 *       異常時 NAN
 */
double compassGetDirection(Compass* compass);

/*
 * 旋回角を初期化する
 * 引数: コンパスポインタ, 初期値 [rad]
 * 戻値: 正常時 0
 *       異常時 -1
 */
int compassSetDirection(Compass* compass,double angle);

/*
 * コンパスの校正を開始する
 * 引数: コンパスポインタ
 * 戻値: 正常時 0
 *       異常時 -1
 */
int compassStartCalibration(Compass* compass);

/*
 * コンパスの校正を終了する
 * 引数: コンパスポインタ
 * 戻値: 正常時 0
 *       異常時 -1
 */
int compassStopCalibration(Compass* compass);

/*
 *degreeで値を返す
*/
int compassDegree(Compass* compass);


#endif /* COMPASS_H_ */
