/*
モータを動作させるモジュール
・制御指令値を PWM値に変換し、左右のモータを駆動する。
2014/07/14  by S.Ushimaru
*/

#include <linux/errno.h>
#include <sys/io.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>

#include "device.h"
#include "header/motor.h"

void motor_drive(int u_l, int u_r , int u_f , int u_b){

	int output_duty_r, output_duty_l, output_duty_f, output_duty_b;        //Duty比にしたもの
	int final_output_r, final_output_l, final_output_f, final_output_b;      //最終的に書きこむ値

	int ret_val;
        unsigned char m_data[4];

	int fd_motor;
	fd_motor=device_motor_get();

        output_duty_r = u_r;
        output_duty_l = u_l;
        output_duty_f = u_f;
        output_duty_b = u_b;
        //printf("duty_r=%d  dutry_l=%d\n", output_duty_r, output_duty_l);

        //PWM不感帯を補正する
        if(output_duty_r > 0){
                output_duty_r = output_duty_r + R_SHIFT_B;
        }else if(output_duty_r < 0){
                output_duty_r = output_duty_r - R_SHIFT_F;
        }else{output_duty_r=0;}

        if(output_duty_l > 0){
                output_duty_l = output_duty_l + L_SHIFT_F;
        }else if(output_duty_l < 0){
                output_duty_l = output_duty_l - L_SHIFT_B;
        }else{output_duty_l=0;}

        if(output_duty_f > 0){
                output_duty_f = output_duty_f + F_SHIFT_F;
        }else if(output_duty_f < 0){
                output_duty_f = output_duty_f - F_SHIFT_B;
        }else{output_duty_f=0;}

        if(output_duty_b > 0){
                output_duty_b = output_duty_b + B_SHIFT_F;
        }else if(output_duty_b < 0){
                output_duty_b = output_duty_b - B_SHIFT_B;
        }else{output_duty_b=0;}

        //入力飽和
        output_duty_r = (output_duty_r > 127) ? 127 : output_duty_r;
        output_duty_r = (output_duty_r < -127) ? -127 : output_duty_r;
        output_duty_l = (output_duty_l > 127) ? 127 : output_duty_l;
        output_duty_l = (output_duty_l < -127) ? -127 : output_duty_l;
        output_duty_f = (output_duty_f > 127) ? 127 : output_duty_f;
        output_duty_f = (output_duty_f < -127) ? -127 : output_duty_f;
        output_duty_b = (output_duty_b > 127) ? 127 : output_duty_b;
        output_duty_b = (output_duty_b < -127) ? -127 : output_duty_b;

        double right_vdata, left_vdata, front_vdata, back_vdata;

        right_vdata = fabs(output_duty_r) ;
        final_output_r = (output_duty_r < 0) ? right_vdata + 128 : right_vdata;

        left_vdata = fabs(output_duty_l) ;
        final_output_l = (output_duty_l < 0) ? left_vdata + 128 : left_vdata;

        front_vdata = fabs(output_duty_f) ;
        final_output_f = (output_duty_f < 0) ? front_vdata + 128 : front_vdata;

        back_vdata = fabs(output_duty_b) ;
        final_output_b = (output_duty_b < 0) ? back_vdata + 128 : back_vdata;


        m_data[0]=final_output_l;
        m_data[1]=final_output_f;
        m_data[2]=final_output_r;
        m_data[3]=final_output_b;
        ret_val=write(fd_motor,m_data,4);

}



void motor_go_left(void){
	motor_drive(0, 0 , -POW_F , POW_B);
	printf("go left\n");
}

void motor_go_front(void){
	motor_drive(POW_L, -POW_R , 0 , 0);
	printf("go front\n");
}

void motor_go_right(void){
	motor_drive(0, 0 , POW_F , -POW_B);
	printf("go right\n");
}

void motor_go_back(void){
	motor_drive(-POW_L, POW_R , 0 , 0);
	printf("go back\n");
}

void motor_stop(void){
	motor_drive(0, 0 , 0 , 0);
	printf("stop\n");
	usleep(100);
}

void RIGHT(void){
	motor_go_right();
	usleep(TIME);
	motor_stop();
}

void LEFT(void){
	motor_go_left();
	usleep(TIME);
	motor_stop();
}

void FRONT(void){
	motor_go_front();
	usleep(TIME);
	motor_stop();
}

void BACK(void){
	motor_go_back();
	usleep(TIME);
	motor_stop();
}


