#include <unistd.h> //for open() close() read() 
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <strings.h>
#include <signal.h> //to handle ctrl-c
#include "header/uss.h"

#define BAUDRATE B9600
#define MODEMDEVICE "/dev/ttyACM0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1

int fd;
struct termios oldtio, newtio;
int isrunning = TRUE;

int serialOpen() {
	fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY ); 
	if (fd <0) {
		printf("open error: %s\n",MODEMDEVICE); 
		return -1; 
	}
	tcgetattr(fd,&oldtio); /* save current port settings */	
	bzero(&newtio,  sizeof(newtio));
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	/* set input mode (non-canonical, no echo,...) */
	newtio.c_lflag = 0;
	newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
	newtio.c_cc[VMIN]     = 1;   /* blocking read until 1 chars received */
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd,TCSANOW,&newtio);
	return 0;
}

void serialClose() {
	tcsetattr(fd,TCSANOW,&oldtio); //restore previous port setting
	close(fd);
}

unsigned char serialReadChar() {
	unsigned char result;
	int length=0;
	tcflush(fd, TCIFLUSH); //clear the receiving buffer
	length = read(fd,&result,1); //blocked until reading 1 byte.	
	return result;
}

void interrupt(int sig) { isrunning = FALSE; } //ctrl-c handler

void  uss_get(int *uss) {
	int c, res;
	unsigned char result;
	if(serialOpen() != 0) printf("error");
	signal(SIGINT, interrupt); //set ctrl-c handler
	result = serialReadChar();		
	switch(result){
		case(65):		//A	ussE
			uss[2] = 0;
			break;
		case(66):		//B
			uss[2] = 1;
			break;
		case(67):		//C
			uss[2] = 2;
			break;
		case(68):		//D
			uss[2] = 3;
			break;
		case(69):		//E
			uss[2] = 4;
			break;
		case(70):		//F
			uss[2] = 5;
			break;
		case(71):		//G
			uss[2] = 6;
			break;
		case(72):		//H
			uss[2] = 7;
			break;
		case(73):		//I
			uss[2] = 8;
			break;
		case(74):		//J	ussS
			uss[3] = 0;
			break;
		case(75):		//K
			uss[3] = 1;
			break;
		case(76):		//L
			uss[3] = 2;
			break;
		case(77):		//M
			uss[3] = 3;
			break;
		case(78):		//N
			uss[3] = 4;
			break;
		case(79):		//O
			uss[3] = 5;
			break;
		case(80):		//P
			uss[3] = 6;
			break;
		case(81):		//Q
			uss[3] = 7;
			break;
		case(82):		//R
			uss[3] = 8;
			break;
		case(97):		//a	ussW
			uss[0] = 0;
			break;
		case(98):		//b
			uss[0] = 1;
			break;
		case(99):		//c
			uss[0] = 2;
			break;
		case(100):		//d
			uss[0] = 3;
			break;
		case(101):		//e
			uss[0] = 4;
			break;
		case(102):		//f
			uss[0] = 5;
			break;
		case(103):		//g
			uss[0] = 6;
			break;
		case(104):		//h
			uss[0] = 7;
			break;
		case(105):		//i
			uss[0] = 8;
			break;
		case(106):		//j	ussN
			uss[1] = 0;
			break;
		case(107):		//k
			uss[1] = 1;
			break;
		case(108):		//l
			uss[1] = 2;
			break;
		case(109):		//m
			uss[1] = 3;
			break;
		case(110):		//n
			uss[1] = 4;
			break;
		case(111):		//o
			uss[1] = 5;
			break;
		case(112):		//p
			uss[1] = 6;
			break;
		case(113):		//q
			uss[1] = 7;
			break;
		case(114):		//r
			uss[1] = 8;
			break;
	}
//	printf("uss1 = %d ",uss1);
//	printf("uss2 = %d ",uss2);
//	printf("uss3 = %d ",uss3);
//	printf("uss4 = %d\n",uss4);        	
//	printf("value is %c %d\n",result,result); 
//	serialClose();
}


